import logging

class UserManager:
    def __init__(self, api_client):
        self.api_client = api_client

    def get_users(self):
        endpoint = "/proxy/access/api/v2/users"
        return self.api_client.get(endpoint)["data"]

    def create_user(self, user):
        endpoint = "/proxy/access/api/v2/user"
        payload = {
            "first_name": user["first_name"],
            "last_name": user["last_name"],
            "employee_number": str(user.get("PersonId", "")),
            "group_ids": user.get("group_ids", [])
        }
        response = self.api_client.post(endpoint, payload)
        logging.info(f'Created user {user["first_name"]} {user["last_name"]}: {response}')
        return response

    def deactivate_user(self, user_id):
        endpoint = f'/proxy/access/ulp-go/api/v2/user/{user_id}/deactivate?isULP=1'
        response = self.api_client.put(endpoint)
        logging.info(f'Deactivated user {user_id}: {response}')
        return response

    def activate_user(self, user_id):
        endpoint = f'/proxy/access/ulp-go/api/v2/user/{user_id}/active?isULP=1'
        response = self.api_client.put(endpoint)
        logging.info(f'Activated user {user_id}: {response}')
        return response

    def delete_user(self, user_id):
        endpoint = f'/proxy/access/ulp-go/api/v2/user/{user_id}?isULP=1'
        response = self.api_client.delete(endpoint)
        logging.info(f'Deleted user {user_id}: {response}')
        return response

    def set_user_group(self, user_id, group_id):
        endpoint = f"/proxy/access/api/v2/user/{user_id}"
        payload = {"group_ids": [group_id]}
        response = self.api_client.put(endpoint, payload)
        logging.info(f"Updated user group for {user_id}")
        return response



import requests
from .users import UserManager
from .hardware import HardwareManager
from .utils import configure_logging

configure_logging()

class UnipyAccess:
    def __init__(self, base_url, username, password, verify=True):
        self.base_url = base_url
        self.username = username
        self.password = password
        self.verify = verify
        self.session = requests.Session()
        self.csrf_token = None
        self._login()

        # Initialize managers
        self.users = UserManager(self)
        self.hardware = HardwareManager(self)

    def _login(self):
        if not self.verify:
            requests.packages.urllib3.disable_warnings(
                requests.packages.urllib3.exceptions.InsecureRequestWarning
            )

        login_url = f"{self.base_url}/api/auth/login"
        login_payload = {
            "username": self.username,
            "password": self.password,
            "token": "",
            "rememberMe": False,
        }
        login_headers = {
            'Content-Type': 'application/json',
            'Origin': self.base_url,
        }

        response = self.session.post(
            login_url,
            headers=login_headers,
            json=login_payload,
            verify=self.verify,
        )

        if response.status_code != 200:
            raise AuthenticationError(f"Login failed: {response.text}")

        self.csrf_token = response.headers.get('x-csrf-token')
        if not self.csrf_token:
            raise AuthenticationError("x-csrf-token not found in login response")

        # Update session headers
        self.session.headers.update({
            'Accept': 'application/json, text/plain, */*',
            'Content-Type': 'application/json',
            'Origin': self.base_url,
            'x-csrf-token': self.csrf_token,
        })

    def request(self, method, endpoint, **kwargs):
        url = f"{self.base_url}{endpoint}"
        response = self.session.request(method, url, verify=self.verify, **kwargs)
        response.raise_for_status()
        return response.json()
    
    def get(self, endpoint):
        return self.request('GET', endpoint)
    
    def post(self, endpoint, payload):
        return self.request('POST', endpoint, json=payload)
    
    def put(self, endpoint, data=None):
        return self.request('PUT', endpoint, json=data)
    
    def delete(self, endpoint):
        return self.request('DELETE', endpoint)



import logging
import sys

class HardwareManager:
    def __init__(self, api_client):
        self.api_client = api_client

    def get_devices(self):
        endpoint = "/proxy/access/api/v2/devices"
        return self.api_client.get(endpoint)
    
    def get_device(self, device_id):
        endpoint = f"/proxy/access/api/v2/device/{device_id}"
        return self.api_client.get(endpoint)

    def get_device_capabilities(self, device_id):
        device_capabilities = self.get_device(device_id)['data']['capabilities']
        return device_capabilities
    
    def get_device_model(self, device_id):
        device_model = self.get_device(device_id)['data']['device_type']
        return device_model
    
    def restart_device(self, device_id):
        endpoint = f"/proxy/access/api/v2/device/{device_id}/restart"
        return self.api_client.post(endpoint)
    
    def set_access_method(self, device_id, enabled_methods):
        endpoint = f"/proxy/access/api/v2/device/{device_id}/configs"
        payload = []

        methods_map = {
            "nfc": "nfc",
            "wave": "wave",
            "mobile_button": "bt_button",
            "mobile_tap": "bt_tap",
            "pin": "pin_code",
        }

        for method, key in methods_map.items():
            value = "yes" if method in enabled_methods else "no"
            payload.append({"key": key, "tag": "open_door_mode", "value": value})

        if "nfc" in enabled_methods and "wave" in enabled_methods:
            logging.error("Cannot enable both NFC and Wave")
            sys.exit(1)
        
        device_capabilities = self.get_device_capabilities(device_id)

        if "pin" in enabled_methods and "pin_code" not in device_capabilities:
            logging.error("PIN is not supported on this device")
            sys.exit(1)

        if "wave" in enabled_methods and "sensitivity" not in device_capabilities:
            logging.error("Wave is not supported on this device")
            sys.exit(1)

        response = self.api_client.put(endpoint, payload)
        logging.info(f"Set access method for device {device_id} to {enabled_methods}")
        return response

    def set_doorbell_trigger(self, device_id, doorbell_trigger):

        device_capabilities = self.get_device_capabilities(device_id)

        if "hold_to_call" not in device_capabilities:
            logging.error("Doorbell trigger is not supported on this device")
            sys.exit(1)

        valid_triggers = ["swipe", "tap", "off"]
        if doorbell_trigger not in valid_triggers:
            logging.error(f"Invalid doorbell trigger: {doorbell_trigger}. Must be one of {valid_triggers}")
            sys.exit(1)
    
        endpoint = f"/proxy/access/api/v2/device/{device_id}/configs"
        payload = [{"tag":"device_setting","key":"door_bell_trigger_type","value":doorbell_trigger}]
        response = self.api_client.put(endpoint, payload)
        logging.info(f"Set doorbell trigger for device {device_id} to {doorbell_trigger}")
        return response
    
    def set_status_light(self, device_id, status_light):
        brightness = None

        # Validate input
        if status_light not in ["off", "on"]:
            logging.error(f"Invalid status light: {status_light}. Must be one of ['off', 'on']")
            sys.exit(1)

        device_model = self.get_device_model(device_id)

        if status_light == "on":
            brightness = 20
        elif status_light == "off":
            brightness = 0
            
        if status_light == "on" and device_model == "UA-G2-PRO":
            brightness = "yes"
        elif status_light == "off" and device_model == "UA-G2-PRO":
            brightness = "no"

        endpoint = f"/proxy/access/api/v2/device/{device_id}/configs"

        if device_model == "UA-G2-PRO":
            payload = [{"key": "status_led", "tag": "device_setting", "value": str(brightness)}]
        else:
            payload = [{"key": "brightness", "tag": "device_setting", "value": str(brightness)}]

        # Send request to API
        response = self.api_client.put(endpoint, payload)
        logging.info(f"Set status light for device {device_id} to {status_light}")

        return response
    
    def set_status_sound(self, device_id, status_sound):
        sound = None
        device_model = self.get_device_model(device_id)

        if device_model == "UA-G2-PRO":
            if isinstance(status_sound, int) and 0 <= status_sound <= 100:
                sound = status_sound
            else:
                logging.error(f"Invalid status sound: {status_sound}. For device model {device_model} value must be a number between 0 and 100.")
                return
        else:
            if status_sound == "on":
                sound = "yes"
            elif status_sound == "off":
                sound = "no"
            else:
                logging.error(f"Invalid status sound: {status_sound}. Must be one of ['off', 'on']")
                return

        endpoint = f"/proxy/access/api/v2/device/{device_id}/configs"
        payload = [{"key": "volume", "tag": "device_setting", "value": str(sound)}]
        response = self.api_client.put(endpoint, payload)
        logging.info(f"Set status sound for device {device_id} to {status_sound}")
        return response

    def set_display_brightness(self, device_id, brightness):
        device_model = self.get_device_model(device_id)

        unsupported_models = ["UA-G2-MINI"]
        if device_model in unsupported_models:
            logging.error(f"Display brightness is not supported on this device")
            sys.exit(1)

        if not isinstance(brightness, int) or not 0 <= brightness <= 100:
            logging.error(f"Invalid display brightness: {brightness}. Must be a number between 0 and 100.")
            sys.exit(1)

        endpoint = f"/proxy/access/api/v2/device/{device_id}/configs"
        payload = [{"key": "brightness", "tag": "device_setting", "value": str(brightness)}]
        response = self.api_client.put(endpoint, payload)
        logging.info(f"Set display brightness for device {device_id} to {brightness}")
        return response


from setuptools import setup, find_packages

with open('README.md', 'r') as f:
    long_description = f.read()

setup(
    name='unipyAccess',
    version='0.2.1',  # Placeholder for the version
    packages=find_packages(),
    install_requires=[
        'requests==2.32.3'
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
)
